"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.invokeNew = exports.wrapAngularDevkitSchematic = exports.runMigration = exports.generate = exports.run = void 0;
const tslib_1 = require("tslib");
const architect_1 = require("@angular-devkit/architect");
const node_1 = require("@angular-devkit/architect/node");
const core_1 = require("@angular-devkit/core");
const node_2 = require("@angular-devkit/core/node");
const params_1 = require("../shared/params");
const run_1 = require("./run");
const tools_1 = require("@angular-devkit/schematics/tools");
const schematics_1 = require("@angular-devkit/schematics");
const inquirer = require("inquirer");
const detect_package_manager_1 = require("../shared/detect-package-manager");
const generate_1 = require("./generate");
const workspace_1 = require("@nrwl/tao/src/shared/workspace");
const workflow_1 = require("@angular-devkit/schematics/src/workflow");
const options_1 = require("@angular-devkit/schematics/tasks/package-manager/options");
const node_3 = require("@angular-devkit/schematics/tasks/node");
const path_1 = require("path");
const fs_1 = require("fs");
const stripJsonComments = require("strip-json-comments");
function normalizeOptions(opts, schema) {
    return params_1.convertAliases(params_1.coerceTypesInOptions(opts, schema), schema, false);
}
function run(logger, root, opts) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const fsHost = new node_2.NodeJsSyncHost();
        const { workspace } = yield core_1.workspaces.readWorkspace(workspace_1.workspaceConfigName(root), core_1.workspaces.createWorkspaceHost(fsHost));
        const registry = new core_1.json.schema.CoreSchemaRegistry();
        registry.addPostTransform(core_1.schema.transforms.addUndefinedDefaults);
        const architectHost = new node_1.WorkspaceNodeModulesArchitectHost(workspace, root);
        const architect = new architect_1.Architect(architectHost, registry);
        const builderConf = yield architectHost.getBuilderNameForTarget({
            project: opts.project,
            target: opts.target,
        });
        const builderDesc = yield architectHost.resolveBuilder(builderConf);
        const flattenedSchema = yield registry
            .flatten(builderDesc.optionSchema)
            .toPromise();
        if (opts.help) {
            run_1.printRunHelp(opts, flattenedSchema, logger);
            return 0;
        }
        const runOptions = normalizeOptions(opts.runOptions, flattenedSchema);
        const run = yield architect.scheduleTarget({
            project: opts.project,
            target: opts.target,
            configuration: opts.configuration,
        }, runOptions, { logger });
        const result = yield run.output.toPromise();
        yield run.stop();
        return result.success ? 0 : 1;
    });
}
exports.run = run;
function createWorkflow(fsHost, root, opts) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const workflow = new tools_1.NodeWorkflow(fsHost, {
            force: opts.force,
            dryRun: opts.dryRun,
            packageManager: detect_package_manager_1.detectPackageManager(),
            root: core_1.normalize(root),
            registry: new core_1.schema.CoreSchemaRegistry(schematics_1.formats.standardFormats),
            resolvePaths: [process.cwd(), root],
        });
        const _params = opts.generatorOptions._;
        delete opts.generatorOptions._;
        workflow.registry.addSmartDefaultProvider('argv', (schema) => {
            if ('index' in schema) {
                return _params[Number(schema['index'])];
            }
            else {
                return _params;
            }
        });
        if (opts.defaults) {
            workflow.registry.addPreTransform(core_1.schema.transforms.addUndefinedDefaults);
        }
        else {
            workflow.registry.addPostTransform(core_1.schema.transforms.addUndefinedDefaults);
        }
        workflow.engineHost.registerOptionsTransform(tools_1.validateOptionsWithSchema(workflow.registry));
        if (opts.interactive !== false && isTTY()) {
            workflow.registry.usePromptProvider((definitions) => {
                const questions = definitions.map((definition) => {
                    const question = {
                        name: definition.id,
                        message: definition.message,
                        default: definition.default,
                    };
                    const validator = definition.validator;
                    if (validator) {
                        question.validate = (input) => validator(input);
                    }
                    switch (definition.type) {
                        case 'confirmation':
                            question.type = 'confirm';
                            break;
                        case 'list':
                            question.type = definition.multiselect ? 'checkbox' : 'list';
                            question.choices =
                                definition.items &&
                                    definition.items.map((item) => {
                                        if (typeof item == 'string') {
                                            return item;
                                        }
                                        else {
                                            return {
                                                name: item.label,
                                                value: item.value,
                                            };
                                        }
                                    });
                            break;
                        default:
                            question.type = definition.type;
                            break;
                    }
                    return question;
                });
                return inquirer.prompt(questions);
            });
        }
        return workflow;
    });
}
function getCollection(workflow, name) {
    const collection = workflow.engine.createCollection(name);
    if (!collection)
        throw new Error(`Cannot find collection '${name}'`);
    return collection;
}
function createRecorder(record, logger) {
    return (event) => {
        const eventPath = event.path.startsWith('/')
            ? event.path.substr(1)
            : event.path;
        if (event.kind === 'error') {
            record.error = true;
            logger.warn(`ERROR! ${eventPath} ${event.description == 'alreadyExist'
                ? 'already exists'
                : 'does not exist.'}.`);
        }
        else if (event.kind === 'update') {
            record.loggingQueue.push(core_1.tags.oneLine `${core_1.terminal.white('UPDATE')} ${eventPath} (${event.content.length} bytes)`);
        }
        else if (event.kind === 'create') {
            record.loggingQueue.push(core_1.tags.oneLine `${core_1.terminal.green('CREATE')} ${eventPath} (${event.content.length} bytes)`);
        }
        else if (event.kind === 'delete') {
            record.loggingQueue.push(`${core_1.terminal.yellow('DELETE')} ${eventPath}`);
        }
        else if (event.kind === 'rename') {
            record.loggingQueue.push(`${core_1.terminal.blue('RENAME')} ${eventPath} => ${event.to}`);
        }
    };
}
function getSchematicDefaults(root, collection, schematic) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const workspace = yield new core_1.experimental.workspace.Workspace(core_1.normalize(root), new node_2.NodeJsSyncHost())
            .loadWorkspaceFromHost(workspace_1.workspaceConfigName(root))
            .toPromise();
        let result = {};
        if (workspace.getSchematics()) {
            const schematicObject = workspace.getSchematics()[`${collection}:${schematic}`];
            if (schematicObject) {
                result = Object.assign(Object.assign({}, result), schematicObject);
            }
            const collectionObject = workspace.getSchematics()[collection];
            if (typeof collectionObject == 'object' &&
                !Array.isArray(collectionObject)) {
                result = Object.assign(Object.assign({}, result), collectionObject[schematic]);
            }
        }
        return result;
    });
}
function runSchematic(root, workflow, logger, opts, schematic, allowAdditionalArgs = false, printDryRunMessage = true, recorder = null) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const flattenedSchema = (yield workflow.registry
            .flatten(schematic.description.schemaJson)
            .toPromise());
        if (opts.help) {
            generate_1.printGenHelp(opts, flattenedSchema, logger);
            return { status: 0, loggingQueue: [] };
        }
        const defaults = opts.generatorName === 'new'
            ? {}
            : yield getSchematicDefaults(root, opts.collectionName, opts.generatorName);
        const record = { loggingQueue: [], error: false };
        workflow.reporter.subscribe(recorder || createRecorder(record, logger));
        const schematicOptions = normalizeOptions(opts.generatorOptions, flattenedSchema);
        if (schematicOptions['--'] && !allowAdditionalArgs) {
            schematicOptions['--'].forEach((unmatched) => {
                const message = `Could not match option '${unmatched.name}' to the ${opts.collectionName}:${opts.generatorName} schema.` +
                    (unmatched.possible.length > 0
                        ? ` Possible matches : ${unmatched.possible.join()}`
                        : '');
                logger.fatal(message);
            });
            return { status: 1, loggingQueue: [] };
        }
        yield workflow
            .execute({
            collection: opts.collectionName,
            schematic: opts.generatorName,
            options: Object.assign(Object.assign({}, defaults), schematicOptions),
            debug: opts.debug,
            logger,
        })
            .toPromise();
        if (!record.error) {
            record.loggingQueue.forEach((log) => logger.info(log));
        }
        if (opts.dryRun && printDryRunMessage) {
            logger.warn(`\nNOTE: The "dryRun" flag means no changes were made.`);
        }
        return { status: 0, loggingQueue: record.loggingQueue };
    });
}
function isTTY() {
    return !!process.stdout.isTTY && process.env['CI'] !== 'true';
}
class MigrationEngineHost extends tools_1.NodeModulesEngineHost {
    constructor(logger) {
        super();
        this.nodeInstallLogPrinted = false;
        // Overwrite the original CLI node package executor with a new one that does basically nothing
        // since nx migrate doesn't do npm installs by itself
        // (https://github.com/angular/angular-cli/blob/5df776780deadb6be5048b3ab006a5d3383650dc/packages/angular_devkit/schematics/tools/workflow/node-workflow.ts#L41)
        this.registerTaskExecutor({
            name: options_1.NodePackageName,
            create: () => Promise.resolve(() => {
                return new Promise((res) => {
                    if (!this.nodeInstallLogPrinted) {
                        logger.warn(`An installation of node_modules has been required. Make sure to run it after the migration`);
                        this.nodeInstallLogPrinted = true;
                    }
                    res();
                });
            }),
        });
        this.registerTaskExecutor(node_3.BuiltinTaskExecutor.RunSchematic);
    }
    _resolveCollectionPath(name) {
        let collectionPath = undefined;
        if (name.startsWith('.') || name.startsWith('/')) {
            name = path_1.resolve(name);
        }
        if (path_1.extname(name)) {
            collectionPath = require.resolve(name);
        }
        else {
            const packageJsonPath = require.resolve(path_1.join(name, 'package.json'));
            // eslint-disable-next-line @typescript-eslint/no-var-requires
            const packageJson = require(packageJsonPath);
            let pkgJsonSchematics = packageJson['nx-migrations'];
            if (!pkgJsonSchematics) {
                pkgJsonSchematics = packageJson['ng-update'];
                if (!pkgJsonSchematics) {
                    throw new Error(`Could not find migrations in package: "${name}"`);
                }
            }
            if (typeof pkgJsonSchematics != 'string') {
                pkgJsonSchematics = pkgJsonSchematics.migrations;
            }
            collectionPath = path_1.resolve(path_1.dirname(packageJsonPath), pkgJsonSchematics);
        }
        try {
            if (collectionPath) {
                JSON.parse(stripJsonComments(fs_1.readFileSync(collectionPath).toString()));
                return collectionPath;
            }
        }
        catch (e) {
            throw new Error(`Invalid migration file in package: "${name}"`);
        }
        throw new Error(`Collection cannot be resolved: "${name}"`);
    }
}
class MigrationsWorkflow extends workflow_1.BaseWorkflow {
    constructor(host, logger) {
        super({
            host,
            engineHost: new MigrationEngineHost(logger),
            force: true,
            dryRun: false,
        });
    }
}
function generate(logger, root, opts) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const fsHost = new core_1.virtualFs.ScopedHost(new node_2.NodeJsSyncHost(), core_1.normalize(root));
        const workflow = yield createWorkflow(fsHost, root, opts);
        const collection = getCollection(workflow, opts.collectionName);
        const schematic = collection.createSchematic(opts.generatorName, true);
        return (yield runSchematic(root, workflow, logger, Object.assign(Object.assign({}, opts), { generatorName: schematic.description.name }), schematic)).status;
    });
}
exports.generate = generate;
function runMigration(logger, root, collection, schematic) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const host = new core_1.virtualFs.ScopedHost(new node_2.NodeJsSyncHost(), core_1.normalize(root));
        const workflow = new MigrationsWorkflow(host, logger);
        return workflow
            .execute({
            collection,
            schematic,
            options: {},
            debug: false,
            logger,
        })
            .toPromise();
    });
}
exports.runMigration = runMigration;
function wrapAngularDevkitSchematic(collectionName, generatorName) {
    return (generatorOptions) => {
        return (host) => tslib_1.__awaiter(this, void 0, void 0, function* () {
            const emptyLogger = {
                log: (e) => { },
                info: (e) => { },
                warn: (e) => { },
                error: (e) => { },
                fatal: (e) => { },
            };
            emptyLogger.createChild = () => emptyLogger;
            const recorder = (event) => {
                const eventPath = event.path.startsWith('/')
                    ? event.path.substr(1)
                    : event.path;
                if (event.kind === 'error') {
                }
                else if (event.kind === 'update') {
                    host.write(eventPath, event.content);
                }
                else if (event.kind === 'create') {
                    host.write(eventPath, event.content);
                }
                else if (event.kind === 'delete') {
                    host.delete(eventPath);
                }
                else if (event.kind === 'rename') {
                    host.rename(eventPath, event.to);
                }
            };
            const fsHost = new core_1.virtualFs.ScopedHost(new node_2.NodeJsSyncHost(), core_1.normalize(host.root));
            yield Promise.all(host.listChanges().map((c) => tslib_1.__awaiter(this, void 0, void 0, function* () {
                if (c.type === 'CREATE' || c.type === 'UPDATE') {
                    yield fsHost.write(c.path, c.content).toPromise();
                }
                else {
                    yield fsHost.delete(c.path).toPromise();
                }
            })));
            const options = {
                generatorOptions: Object.assign(Object.assign({}, generatorOptions), { _: [] }),
                dryRun: true,
                interactive: false,
                help: false,
                debug: false,
                collectionName,
                generatorName,
                force: false,
                defaults: false,
            };
            const workflow = yield createWorkflow(fsHost, host.root, options);
            const collection = getCollection(workflow, collectionName);
            const schematic = collection.createSchematic(generatorName, true);
            const res = yield runSchematic(host.root, workflow, emptyLogger, options, schematic, false, false, recorder);
            if (res.status !== 0) {
                throw new Error(res.loggingQueue.join('\n'));
            }
        });
    };
}
exports.wrapAngularDevkitSchematic = wrapAngularDevkitSchematic;
function invokeNew(logger, root, opts) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const fsHost = new core_1.virtualFs.ScopedHost(new node_2.NodeJsSyncHost(), core_1.normalize(root));
        const workflow = yield createWorkflow(fsHost, root, opts);
        const collection = getCollection(workflow, opts.collectionName);
        const schematic = collection.createSchematic('new', true);
        const allowAdditionalArgs = true; // we can't yet know the schema to validate against
        return (yield runSchematic(root, workflow, logger, Object.assign(Object.assign({}, opts), { generatorName: schematic.description.name }), schematic, allowAdditionalArgs)).status;
    });
}
exports.invokeNew = invokeNew;
//# sourceMappingURL=ngcli-adapter.js.map