/// <reference types="node" />
import { ParsedArgs } from 'minimist';
import { TargetConfiguration, WorkspaceConfiguration } from './workspace';
declare type Properties = {
    [p: string]: {
        type?: string;
        properties?: any;
        oneOf?: any;
        items?: any;
        alias?: string;
        description?: string;
        default?: string | number | boolean | string[];
        $default?: {
            $source: 'argv';
            index: number;
        };
        'x-prompt'?: string | {
            message: string;
            type: string;
            items: any[];
        };
    };
};
export declare type Schema = {
    properties: Properties;
    required?: string[];
    description?: string;
};
export declare type Unmatched = {
    name: string;
    possible: string[];
};
export declare type Options = {
    '--'?: Unmatched[];
    [k: string]: string | number | boolean | string[] | Unmatched[];
};
export declare function handleErrors(logger: Console, isVerbose: boolean, fn: Function): Promise<any>;
export declare function convertToCamelCase(parsed: ParsedArgs): Options;
/**
 * Coerces (and replaces) options identified as 'boolean' or 'number' in the Schema
 *
 * @param opts The options to check
 * @param schema The schema definition with types to check against
 *
 */
export declare function coerceTypesInOptions(opts: Options, schema: Schema): Options;
/**
 * Converts any options passed in with short aliases to their full names if found
 * Unmatched options are added to opts['--']
 *
 * @param opts The options passed in by the user
 * @param schema The schema definition to check against
 */
export declare function convertAliases(opts: Options, schema: Schema, excludeUnmatched: boolean): Options;
export declare class SchemaError {
    readonly message: string;
    constructor(message: string);
}
export declare function validateOptsAgainstSchema(opts: {
    [k: string]: any;
}, schema: Schema): void;
export declare function validateObject(opts: {
    [k: string]: any;
}, properties: Properties, required: string[]): void;
export declare function setDefaults(opts: {
    [k: string]: any;
}, schema: Schema): {
    [k: string]: any;
};
export declare function convertPositionParamsIntoNamedParams(opts: {
    [k: string]: any;
}, schema: Schema, argv: string[]): void;
export declare function combineOptionsForExecutor(commandLineOpts: Options, config: string, target: TargetConfiguration, schema: Schema): any;
export declare function combineOptionsForGenerator(commandLineOpts: Options, collectionName: string, generatorName: string, ws: WorkspaceConfiguration, schema: Schema, isInteractive: boolean): Promise<Options>;
/**
 * Tries to find what the user meant by unmatched commands
 *
 * @param opts The options passed in by the user
 * @param schema The schema definition to check against
 *
 */
export declare function lookupUnmatched(opts: Options, schema: Schema): Options;
export {};
